%    <CustomTools>
%      <Menu>
%       <Submenu name="Surfaces Functions">
%        <Item name="dilate Surface" icon="Matlab">
%          <Command>MatlabXT::dilateSurface3(%i)</Command>
%        </Item>
%       </Submenu>
%      </Menu>
%      <SurpassTab>
%        <SurpassComponent name="bpSurfaces">
%          <Item name="dilate Surface" icon="Matlab">
%            <Command>MatlabXT::dilateSurface3(%i)</Command>
%          </Item>
%        </SurpassComponent>
%      </SurpassTab>
%    </CustomTools>







function [] = dilateSurface3(aImarisApplicationID)
%UNTITLED3 Summary of this function goes here
%   Detailed explanation goes here


if ~isnumeric(aImarisApplicationID)
    aImarisApplicationID=str2num(aImarisApplicationID);
end

if ~isnumeric(aImarisApplicationID) && ~ischar(aImarisApplicationID)
    aImarisApplicationID=0;
end
global vImarisApplication;

if ~isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
  javaaddpath ImarisLib.jar
  vImarisLib = ImarisLib;
  if ischar(aImarisApplicationID)
    aImarisApplicationID = round(str2double(aImarisApplicationID));
  end
   vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
else
  vImarisApplication = aImarisApplicationID;
end

vDataSet = vImarisApplication.GetDataSet.Clone;
vSurpassScene = vImarisApplication.GetSurpassScene;
aType = Imaris.tType.eTypeFloat;

if strcmp(vDataSet.GetType,'eTypeUInt8')
    vImarisApplication.GetDataSet.SetType(aType);
elseif strcmp(vDataSet.GetType,'eTypeUInt16')
    vImarisApplication.GetDataSet.SetType(aType);
end    
vDataSet = vImarisApplication.GetDataSet.Clone;



%Get DataSet Info for voxel sizing etc...
vDataMin = [vDataSet.GetExtendMinX, vDataSet.GetExtendMinY, vDataSet.GetExtendMinZ];
vDataMax = [vDataSet.GetExtendMaxX, vDataSet.GetExtendMaxY, vDataSet.GetExtendMaxZ];
vDataSize = [vDataSet.GetSizeX, vDataSet.GetSizeY, vDataSet.GetSizeZ];

vTimeMax = vDataSet.GetSizeT - 1;

SizeX = vDataSet.GetSizeX;
SizeY = vDataSet.GetSizeY;
SizeZ = vDataSet.GetSizeZ;
PixelSize = (vDataSet.GetExtendMaxX)/SizeX;
PixelDepth = (vDataSet.GetExtendMaxZ)/SizeZ;
vNumberOfChannels = vDataSet.GetSizeC;

%Get Selected Surface, or ask user to choose a surface object from list
vDataItem = vImarisApplication.GetSurpassSelection;
nSurfaces=0;
surfSelected=0;
if ~isempty(vDataItem)  
    if ~vImarisApplication.GetFactory.IsSurfaces(vDataItem)
        vDataItem=[];
    end
end    
DialogTitle='Please choose a surface object';
if isempty(vDataItem)  
    for vChildIndex = 1:vSurpassScene.GetNumberOfChildren
        vDataItem = vSurpassScene.GetChild(vChildIndex - 1);
        if vImarisApplication.GetFactory.IsSurfaces(vDataItem)
            nSurfaces=nSurfaces+1;
            vSurfaces{nSurfaces} = vImarisApplication.GetFactory.ToSurfaces(vDataItem);    
            SurfNames{nSurfaces}=char(vSurfaces{nSurfaces}.GetName);
        end
    end    
else
    if vImarisApplication.GetFactory.IsSurfaces(vDataItem)
        vSurface=vImarisApplication.GetFactory.ToSurfaces(vDataItem);
        SurfName{1}=char(vSurface.GetName);
        nSurfaces=1;
        surfSelected=1;
    end     
end    
if surfSelected==0
  choice = menu(DialogTitle,SurfNames);
  vSurface = vSurfaces{choice};
end

  
%do Distance Transform
modXTDistanceTransform(vSurface,1,0,0,0,vImarisApplication)

%Get distance Transform channel back from Imaris and use the zeros(original
%surface regions) as the starting point for the watershed function in order
%split expanded regions.
for vTime=0:vTimeMax
    [dTransCH{vTime+1},ch1voxelsize] = XTGetImarisImage(vImarisApplication,vTime+1,vNumberOfChannels+1);
    oldSurfMask{vTime+1}=dTransCH{vTime+1}==0;
    L{vTime+1}=watershed(~oldSurfMask{vTime+1},26);
    dTransCH{vTime+1}(L{vTime+1}==0)=Inf; %set to INF since <= operation is used for mask
end    

%voxel depth (Z scaling) is the minimum recomended expansion size
userChar=inputdlg(['enter the distance you want to expand the surfaces by:(um) minimum of '...
    num2str(PixelDepth) ' recommended'],'Expand BY:');
userVal= str2double(userChar{1});
if ~isnumeric(userVal) || isnan(userVal) || lt(userVal,0)
    display('please input only a positive number in this field!');
    userChar=inputdlg('enter the distance you want to expand the surfaces by:(um)   ONLY NUMBERS PLEASE','Expand BY:');
    userVal= str2double(userChar{1});   
end    
if ~isnumeric(userVal) || isnan(userVal)
    error('closing...bad user input');
end    


choices={'original surface + expanded ring','expanded ring ONLY'};
choice2 = menu('choose final surface object type',choices);



for vTime=0:vTimeMax
    newSurfMask{vTime+1}=dTransCH{vTime+1}<=userVal;
end
if choice2==1
    SurfName=['Expanded ' char(vSurface.GetName)];
end    

%get rid of unnecessary split regions that do not include a surface from
%the original object
for vTime=0:vTimeMax
    newSurfMaskStats{vTime+1}=regionprops(bwlabeln(newSurfMask{vTime+1}),'PixelIdxList');
    [nSurfsHere,dontcare]=size(newSurfMaskStats{vTime+1});
    for i=1:nSurfsHere
        npixOfSurfInROI{vTime+1,i}=find(dTransCH{vTime+1}(newSurfMaskStats{vTime+1}(i).PixelIdxList)==0);
        if isempty(npixOfSurfInROI{vTime+1,i})
            newSurfMask{vTime+1}(newSurfMaskStats{vTime+1}(i).PixelIdxList)=0;
        end
    end
end

%set original mask to zero to exclude from final region if 'ring' is chosen
if choice2==2
    SurfName=['Ring around ' char(vSurface.GetName)];
    for vTime=0:vTimeMax
        newSurfMask{vTime+1}(dTransCH{vTime+1}==0)=0;
    end
end




vDataSet = vImarisApplication.GetDataSet.Clone;
vNumberOfChannels = vDataSet.GetSizeC;


for vTime=0:vTimeMax
    XTSetImarisImage(vImarisApplication,vDataSet,newSurfMask{vTime+1},vTime+1,vNumberOfChannels);
end    


choice3=menu('do you want to create a new surface for the new channel','Create Surface For Me','I will create surface myself');

if choice3==1
    vDataSet = vImarisApplication.GetDataSet.Clone;
    ROI=[];
    ChannelIndex=vNumberOfChannels-1;
    SmoothFilterWidth=0;
    lclCntrstFiltWidth=0;
    boolThreshAuto=0;
    IntensityThreshold=0.001;
    FilterStringAuto='';
    %%pushing Imaris too fast here, Inserting a pause for 12
    %%seconds...otherwise Imaris will crash
    smValue=inputdlg('Enter smoothing Value for new expanded surface (0=binary result is recommended for better roi splitting)      ','smoothing Value');
    display('please wait a moment while surface object is created');
    pause(12)%%pushing Imaris too fast here
    ExpandedSurfaces = MyCreateSurface(vImarisApplication,SurfName,vDataSet,ROI,ChannelIndex,...
        str2num(smValue{1}),lclCntrstFiltWidth,boolThreshAuto,IntensityThreshold,FilterStringAuto);
else
    display(sprintf('please create a surface using the newly added channel %s',SurfName));
    pause(5);
end    






stophereplease=3;

end




function [vSurfaces] = MyCreateSurface(vImarisApplication,SurfName,vDataSet,ROI,...
    ChannelIndex,SmoothFilterWidth,lclCntrstFiltWidth,boolThreshAuto,...
    IntensityThreshold,FilterStringAuto)

% This function creates a new surface object and places in the surpass
% scene. also, returns the surface object created..
% so must be used like:



vSurpassScene = vImarisApplication.GetSurpassScene;
vSurfaces = vImarisApplication.GetFactory.CreateSurfaces;
vSurfaces.SetVisible(1);
vSurfaces = vImarisApplication.GetImageProcessing.DetectSurfaces(vDataSet,...
     ROI,ChannelIndex,SmoothFilterWidth,...
    lclCntrstFiltWidth,boolThreshAuto,IntensityThreshold,FilterStringAuto);
vSurfaces.SetName(SurfName);
vSurfaces.SetColorRGBA(255*255);
vSurpassScene.AddChild(vSurfaces, -1);



end












function [void] = modXTDistanceTransform(ImarisObject,boolOut,boolFillHoles,...
    FillIterations,FilterRadius,vImarisApplication)


% Get Surpass Surfaces Object
vImarisObject = ImarisObject;  
vImarisDataSet = vImarisApplication.GetDataSet.Clone;
% Check if there is a selection
if isempty(vImarisObject)
  msgbox('A spots/surfaces object must be selected');
  return;
end
vDataMin = [vImarisDataSet.GetExtendMinX, vImarisDataSet.GetExtendMinY, vImarisDataSet.GetExtendMinZ];
vDataMax = [vImarisDataSet.GetExtendMaxX, vImarisDataSet.GetExtendMaxY, vImarisDataSet.GetExtendMaxZ];
vDataSize = [vImarisDataSet.GetSizeX, vImarisDataSet.GetSizeY, vImarisDataSet.GetSizeZ];



% if the DataSet is not of type float, display a warning. As the result of
% DistanceTransform will very probably be floating values, a DataSet of
% type int will not be able to display it correctly.
if strcmp(vImarisDataSet.GetType,'eTypeUInt8') || strcmp(vImarisDataSet.GetType,'eTypeUInt16')
  vWarningAnswer = warndlg('Due to the DataSet type the results of the Distance Transform function will probably be truncated! It is best to use it with a float DataSet!','!Warning!');
  waitfor(vWarningAnswer);
end




%if boolFillHoles is true, then create the filter using FilterRadius input
if isequal(boolFillHoles,1)
    r = FilterRadius;
    pixelradius = r;
    r2=r/2;
    loopend=FillIterations;
end    

% Check if the selection is a surfaces object
if vImarisApplication.GetFactory.IsSurfaces(vImarisObject)
  vImarisObject = vImarisApplication.GetFactory.ToSurfaces(vImarisObject);
  vIsSurfaces = true;
elseif vImarisApplication.GetFactory.IsSpots(vImarisObject)
  vImarisObject = vImarisApplication.GetFactory.ToSpots(vImarisObject);
else
  msgbox('Your selection is not a valid spots or surfaces object');
  return;
end


vSelection = boolOut;
vSpotsXYZ = [];
vSpotsTime = [];
iterator = 0;
if vIsSurfaces
    vSelection = boolOut;    
% else the object is a spots object.
else 
  vSpotsValues = vImarisObject.Get;
  vSpotsXYZ = vSpotsValues.mPositionsXYZ;
  vSpotsTime = vSpotsValues.mIndicesT;

  vInd = ones(numel(vSpotsTime), 1);
  for vDim = 1:3
    vSpotsXYZ(:, vDim) = floor((vSpotsXYZ(:, vDim) - vDataMin(vDim)) ./ ...
      (vDataMax(vDim) - vDataMin(vDim)) .* vDataSize(vDim)) + 1;
    vInd = vInd & vSpotsXYZ(:, vDim) > 0 & vSpotsXYZ(:, vDim) <= vDataSize(vDim);
  end
  vSpotsXYZ = vSpotsXYZ(vInd, :);
  vSpotsTime = vSpotsTime(vInd);
end

vProgressDisplay = waitbar(0, 'Distance Transform: Preparation');

% Create a new channel where the result will be sent
vNumberOfChannels = vImarisDataSet.GetSizeC;
vImarisDataSet.SetSizeC(vNumberOfChannels + 1);
vImarisDataSet.SetChannelName(vNumberOfChannels,['Expansion Mask of ', char(vImarisObject.GetName)]);
vImarisDataSet.SetChannelColorRGBA(vNumberOfChannels, 255*256*256);

vSizeT = vImarisDataSet.GetSizeT;
for vTime = 0:vSizeT-1;
  if vIsSurfaces
    
    % Get the mask DataSet
    
        vMaskDataSet = vImarisObject.GetMask( ...
          vDataMin(1), vDataMin(2), vDataMin(3), ...
          vDataMax(1), vDataMax(2), vDataMax(3), ...
          vDataSize(1), vDataSize(2), vDataSize(3), vTime);
        for vIndexZ = 1:vDataSize(3)
          vSlice = vMaskDataSet.GetDataSliceBytes(vIndexZ-1, 0, 0);
          if vSelection == 0
            vSlice = vSlice ~= 1;
          else
            vSlice = vSlice == 1;
          end
          if boolFillHoles
            vSlice = ~vSlice;
            vSlice = imfill(vSlice,'holes');
            vSlice = ~vSlice;
          end    
          if strcmp(vImarisDataSet.GetType,'eTypeUInt8')
            vImarisDataSet.SetDataSliceBytes(vSlice, ...
              vIndexZ-1, vNumberOfChannels, vTime);
          elseif strcmp(vImarisDataSet.GetType,'eTypeUInt16')
            vImarisDataSet.SetDataSliceShorts(vSlice, ...
              vIndexZ-1, vNumberOfChannels, vTime);
          elseif strcmp(vImarisDataSet.GetType,'eTypeFloat')
            vImarisDataSet.SetDataSliceFloats(vSlice, ...
              vIndexZ-1, vNumberOfChannels, vTime);
          end
        end    
  else % is spots
    
    vThisTime = vSpotsTime == vTime;
    vThisSpots = vSpotsXYZ(vThisTime, :);
    for vIndexZ = 1:vDataSize(3)
      vSliceSpots = vThisSpots(vThisSpots(:, 3) == vIndexZ, :);
      if isempty(vSliceSpots)
        continue;
      end
      vSlice = zeros(vDataSize(1), vDataSize(2), 'uint8');
      vSlice(vSliceSpots(:, 1) + (vSliceSpots(:, 2) - 1) * vDataSize(1)) = 1;
      if strcmp(vImarisDataSet.GetType,'eTypeUInt8')
        vImarisDataSet.SetDataSliceBytes(vSlice, ...
          vIndexZ-1, vNumberOfChannels, vTime);
      elseif strcmp(vImarisDataSet.GetType,'eTypeUInt16')
        vImarisDataSet.SetDataSliceShorts(vSlice, ...
          vIndexZ-1, vNumberOfChannels, vTime);
      elseif strcmp(vImarisDataSet.GetType,'eTypeFloat')
        vImarisDataSet.SetDataSliceFloats(vSlice, ...
          vIndexZ-1, vNumberOfChannels, vTime);
      end
    end
    
  end

  waitbar((vTime+1)/vSizeT/2, vProgressDisplay);
end

waitbar(0.5, vProgressDisplay, 'Distance Transform: Calculation');
vImarisApplication.GetImageProcessing.DistanceTransformChannel( ...
  vImarisDataSet, vNumberOfChannels, 1, false);
waitbar(1, vProgressDisplay);

vImarisApplication.SetDataSet(vImarisDataSet);

close(vProgressDisplay);
end



%
%
%  Send Image to Imaris 7.3.0
%
%  Copyright Bitplane AG 2011
%
%
%  Description:
%   
%   Change the Imaris DataSet.
%   This function can be used as utility for other functions; It is not
%       called directly by Imaris.
%
%

function XTSetImarisImage(aImarisApplication, vDataSet, aImage, aTimeIndex, aChannelIndex)


if vDataSet.GetSizeX ~= size(aImage,1)  || vDataSet.GetSizeY ~= size(aImage,2) || vDataSet.GetSizeZ ~= size(aImage,3)
   vDataSet.Resize(0,size(aImage,1), 0,size(aImage,2), 0, size(aImage,3), 0, vDataSet.GetSizeC, 0, vDataSet.GetSizeT);
end
if size(aImage,1) == vDataSet.GetSizeX && ...
        size(aImage,2) == vDataSet.GetSizeY && ...
        size(aImage,3) == vDataSet.GetSizeZ
   if strcmp(vDataSet.GetType,'eTypeUInt8')
        vMin = min(reshape(aImage,[1,numel(aImage)]));
        if vMin < 0
            aImage = aImage + vMin;
        end
        vMax = max(reshape(aImage,[1,numel(aImage)]));
        if vMax > (2^8-1)
            aImage = aImage*(2^8-1)/vMax;
        end
        vDataSet.SetDataVolumeBytes(uint8(aImage), aChannelIndex-1, aTimeIndex-1);
   elseif strcmp(vDataSet.GetType,'eTypeUInt16')
        vMin = min(reshape(aImage,[1,numel(aImage)]));
        if vMin < 0
            aImage = aImage + vMin;
        end
        vMax = max(reshape(aImage,[1,numel(aImage)]));
        if vMax > (2^16-1)
            aImage = aImage*(2^16-1)/vMax;
        end 
        vDataSet.SetDataVolumeShorts(uint16(aImage), aChannelIndex-1, aTimeIndex-1);
   elseif strcmp(vDataSet.GetType,'eTypeFloat')
        vDataSet.SetDataVolumeFloats(single(aImage), aChannelIndex-1, aTimeIndex-1);
   end
end
aImarisApplication.SetDataSet(vDataSet);
end




%
%
%  Get Image from Imaris 7.3.0
%
%  Copyright Bitplane AG 2011
%
%
%  Description:
%   
%   Get and returns the Imaris DataSet.
%   This function can be used as utility for other functions; It is not
%       called directly by Imaris.
%
%

function [aImage, aVoxelSize] = XTGetImarisImage(aImarisApplication, aTimeIndex, aChannelIndex)

vImarisDataSet = aImarisApplication.GetDataSet;

vExtendMin = [vImarisDataSet.GetExtendMinX, vImarisDataSet.GetExtendMinY, ...
  vImarisDataSet.GetExtendMinZ];
vExtendMax = [vImarisDataSet.GetExtendMaxX, vImarisDataSet.GetExtendMaxY, ...
  vImarisDataSet.GetExtendMaxZ];
vDataSize = [vImarisDataSet.GetSizeX, vImarisDataSet.GetSizeY, ...
  vImarisDataSet.GetSizeZ, vImarisDataSet.GetSizeC, vImarisDataSet.GetSizeT];

aVoxelSize = (vExtendMax - vExtendMin) ./ vDataSize(1:3);

vMinT = 0;
vMinC = 0;

if nargin == 3 && aChannelIndex<=vDataSize(4) && aTimeIndex<=vDataSize(5)

  vMinT = aTimeIndex-1;
  vDataSize(5) = 1;
  vMinC = aChannelIndex-1;
  vDataSize(4) = 1;
  
elseif nargin == 2 && aTimeIndex<=vDataSize(5)

  vMinT = aTimeIndex-1;
  vDataSize(5) = 1;

end

vSize = [1, prod(vDataSize(1:3)), vDataSize(4:5)];

if strcmp(vImarisDataSet.GetType,'eTypeUInt8')
  aImage = zeros(vSize, 'uint8');
elseif strcmp(vImarisDataSet.GetType,'eTypeUInt16')
  aImage = zeros(vSize, 'uint16');
else
  aImage = zeros(vSize, 'single');
end

for vTimeIndex = 1:vDataSize(5)
  for vChannelIndex = 1:vDataSize(4)
    if strcmp(vImarisDataSet.GetType,'eTypeUInt8')
      aImage(1, :, vChannelIndex, vTimeIndex) = typecast(...
        vImarisDataSet.GetDataVolumeAs1DArrayBytes( ...
        vMinC + vChannelIndex-1, vMinT + vTimeIndex-1), 'uint8');
    elseif strcmp(vImarisDataSet.GetType,'eTypeUInt16')
      aImage(1, :, vChannelIndex, vTimeIndex) = typecast(...
        vImarisDataSet.GetDataVolumeAs1DArrayShorts( ...
        vMinC + vChannelIndex-1, vMinT + vTimeIndex-1), 'uint16');
    else
      aImage(1, :, vChannelIndex, vTimeIndex) = ...
        vImarisDataSet.GetDataVolumeAs1DArrayFloats( ...
        vMinC + vChannelIndex-1, vMinT + vTimeIndex-1);
    end
  end
end
aImage = reshape(aImage, vDataSize);
end